/*!
    \file TSignature_generated
    \author Kurt Stege
    Generated by the tool \c hpp_generator at Wed Sep 07 16:45:15 CEST 2005
*/

#ifndef NSigslot_TSignature_generated_hpp_INCLUDEGUARD
#define NSigslot_TSignature_generated_hpp_INCLUDEGUARD

/*
History:

(ks) = Kurt Stege, kurt-stege@online.de

2005-07-20 (ks):    File created.
2005-07-28 (ks):    current version, file still in development.
2005-08-29 (ks):    Bugfixes in the constructor and assignment operator of CMessage.
2005-09-07 (ks):    Added function isConnected() to CSignal.
*/

namespace NSigslot {


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.
    \note This case TSignature0<> is special. The signature
    gets zero arguments, alas, in C++ a template with zero
    arguments like TSignature0<> is not valid. Instead, we
    have to remove the empty template argument list, and
    get an ordinary class TSignature0. We still keep the
    name TSignature0 instead of CSignature0 for its similarity
    with all the other signature template classes.

    (An alternative would have been to give the template
    a dummy argument where the user is supposed to enter \c void.
    But for there is no real difference between a template class
    without real arguments and a normal non-template class,
    we choose to provide a normal class and handle it like
    the other template classes.)

*/
class TSignature0
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    enum {numberOfParameters = 0};    //!< number of the parameters of the signal

    typedef TSignature0 tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        // This class is empty on purpose, for a void signal sends a void message without parameters.

        explicit CMessage()
        {
        }
        
        CMessage(const CMessage &source)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()() const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage());
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature0


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1>
class TSignature1
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter

    enum {numberOfParameters = 1};    //!< number of the parameters of the signal

    typedef TSignature1<tType1> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1)
        :
            m_par1(par1)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature1


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2>
class TSignature2
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter

    enum {numberOfParameters = 2};    //!< number of the parameters of the signal

    typedef TSignature2<tType1, tType2> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2)
        :
            m_par1(par1),
            m_par2(par2)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature2


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3>
class TSignature3
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter

    enum {numberOfParameters = 3};    //!< number of the parameters of the signal

    typedef TSignature3<tType1, tType2, tType3> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature3


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4>
class TSignature4
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter

    enum {numberOfParameters = 4};    //!< number of the parameters of the signal

    typedef TSignature4<tType1, tType2, tType3, tType4> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature4


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5>
class TSignature5
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tType5;    //!< type of the 5st/nd/rd/th signal parameter

    enum {numberOfParameters = 5};    //!< number of the parameters of the signal

    typedef TSignature5<tType1, tType2, tType3, tType4, tType5> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal
        tType5 m_par5;    //!< the 5st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4),
            m_par5(par5)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4),
            m_par5(source.m_par5)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            m_par5 = rhs.m_par5;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4, par5));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature5


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6>
class TSignature6
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tType5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tType6;    //!< type of the 6st/nd/rd/th signal parameter

    enum {numberOfParameters = 6};    //!< number of the parameters of the signal

    typedef TSignature6<tType1, tType2, tType3, tType4, tType5, tType6> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal
        tType5 m_par5;    //!< the 5st/nd/rd/th parameter of the signal
        tType6 m_par6;    //!< the 6st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4),
            m_par5(par5),
            m_par6(par6)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4),
            m_par5(source.m_par5),
            m_par6(source.m_par6)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            m_par5 = rhs.m_par5;
            m_par6 = rhs.m_par6;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4, par5, par6));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature6


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7>
class TSignature7
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tType5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tType6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tType7;    //!< type of the 7st/nd/rd/th signal parameter

    enum {numberOfParameters = 7};    //!< number of the parameters of the signal

    typedef TSignature7<tType1, tType2, tType3, tType4, tType5, tType6, tType7> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal
        tType5 m_par5;    //!< the 5st/nd/rd/th parameter of the signal
        tType6 m_par6;    //!< the 6st/nd/rd/th parameter of the signal
        tType7 m_par7;    //!< the 7st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4),
            m_par5(par5),
            m_par6(par6),
            m_par7(par7)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4),
            m_par5(source.m_par5),
            m_par6(source.m_par6),
            m_par7(source.m_par7)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            m_par5 = rhs.m_par5;
            m_par6 = rhs.m_par6;
            m_par7 = rhs.m_par7;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4, par5, par6, par7));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature7


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7, typename TemplateParameter8>
class TSignature8
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tType5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tType6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tType7;    //!< type of the 7st/nd/rd/th signal parameter
    typedef TemplateParameter8 tType8;    //!< type of the 8st/nd/rd/th signal parameter

    enum {numberOfParameters = 8};    //!< number of the parameters of the signal

    typedef TSignature8<tType1, tType2, tType3, tType4, tType5, tType6, tType7, tType8> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal
        tType5 m_par5;    //!< the 5st/nd/rd/th parameter of the signal
        tType6 m_par6;    //!< the 6st/nd/rd/th parameter of the signal
        tType7 m_par7;    //!< the 7st/nd/rd/th parameter of the signal
        tType8 m_par8;    //!< the 8st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7, tType8 par8)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4),
            m_par5(par5),
            m_par6(par6),
            m_par7(par7),
            m_par8(par8)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4),
            m_par5(source.m_par5),
            m_par6(source.m_par6),
            m_par7(source.m_par7),
            m_par8(source.m_par8)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            m_par5 = rhs.m_par5;
            m_par6 = rhs.m_par6;
            m_par7 = rhs.m_par7;
            m_par8 = rhs.m_par8;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7, tType8 par8) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4, par5, par6, par7, par8));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature8


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
	TSignature<n> is the main template class of the Sigslot concept.
	
	TODO: Some detailed description.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7, typename TemplateParameter8, typename TemplateParameter9>
class TSignature9
{
    public:
    
    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tType1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tType2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tType3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tType4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tType5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tType6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tType7;    //!< type of the 7st/nd/rd/th signal parameter
    typedef TemplateParameter8 tType8;    //!< type of the 8st/nd/rd/th signal parameter
    typedef TemplateParameter9 tType9;    //!< type of the 9st/nd/rd/th signal parameter

    enum {numberOfParameters = 9};    //!< number of the parameters of the signal

    typedef TSignature9<tType1, tType2, tType3, tType4, tType5, tType6, tType7, tType8, tType9> tSig;
    
    /*!
        An object of type \c CMessage holds all the parameters of a signal.
        It is used to forward the parameters to the cable functions
        but may be used for other purposes as well.
    */

    class CMessage
    {
        public:
        
        tType1 m_par1;    //!< the 1st/nd/rd/th parameter of the signal
        tType2 m_par2;    //!< the 2st/nd/rd/th parameter of the signal
        tType3 m_par3;    //!< the 3st/nd/rd/th parameter of the signal
        tType4 m_par4;    //!< the 4st/nd/rd/th parameter of the signal
        tType5 m_par5;    //!< the 5st/nd/rd/th parameter of the signal
        tType6 m_par6;    //!< the 6st/nd/rd/th parameter of the signal
        tType7 m_par7;    //!< the 7st/nd/rd/th parameter of the signal
        tType8 m_par8;    //!< the 8st/nd/rd/th parameter of the signal
        tType9 m_par9;    //!< the 9st/nd/rd/th parameter of the signal

        explicit CMessage(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7, tType8 par8, tType9 par9)
        :
            m_par1(par1),
            m_par2(par2),
            m_par3(par3),
            m_par4(par4),
            m_par5(par5),
            m_par6(par6),
            m_par7(par7),
            m_par8(par8),
            m_par9(par9)
        {
        }
        
        CMessage(const CMessage &source)
        :
            m_par1(source.m_par1),
            m_par2(source.m_par2),
            m_par3(source.m_par3),
            m_par4(source.m_par4),
            m_par5(source.m_par5),
            m_par6(source.m_par6),
            m_par7(source.m_par7),
            m_par8(source.m_par8),
            m_par9(source.m_par9)
        {
        }
        
        CMessage &operator= (const CMessage &rhs)
        {
            m_par1 = rhs.m_par1;
            m_par2 = rhs.m_par2;
            m_par3 = rhs.m_par3;
            m_par4 = rhs.m_par4;
            m_par5 = rhs.m_par5;
            m_par6 = rhs.m_par6;
            m_par7 = rhs.m_par7;
            m_par8 = rhs.m_par8;
            m_par9 = rhs.m_par9;
            return *this;
        }
    }; // class CMessage

    /*!
        A cable function is used by a signal to transport the message
        to all the connected receivers.
    */
    typedef void (*tCableFunctionPointer)(void *context, const CMessage &msg);

    /*!
        A signal is used to emit the message for a signal slot connection.
    */
    class CSignal
    {
        public:

        CSignal()
        :
            m_cable(0),
            m_context(0)
        {
        }

        explicit CSignal(const CSignal &source)
        :
            m_cable(source.m_cable),
            m_context(source.m_context)
        {
        }

        explicit CSignal(tCableFunctionPointer cable)
        :
            m_cable(cable),
            m_context(0)
        {
        }

        explicit CSignal(tCableFunctionPointer cable, const void *context)
        :
            m_cable(cable),
            m_context(const_cast<void *>(context))
        {
        }

        ~CSignal()
        {
        }

        CSignal &operator= (const CSignal &rhs)
        {
            m_cable = rhs.m_cable;
            m_context = rhs.m_context;
            return *this;
        }
        
        void operator()(tType1 par1, tType2 par2, tType3 par3, tType4 par4, tType5 par5, tType6 par6, tType7 par7, tType8 par8, tType9 par9) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, CMessage(par1, par2, par3, par4, par5, par6, par7, par8, par9));
            }
        }

        void emitPackedMessage(const CMessage &msg) const
        {
            if (m_cable != 0)
            {
                m_cable(m_context, msg);
            }
        }

        bool isConnected() const
        {
            return m_cable != 0;
        }

        private:

        tCableFunctionPointer m_cable;
        void *m_context;
    };

}; // template class TSignature9


} // namespace NSigslot

#endif // NSigslot_TSignature_generated_hpp_INCLUDEGUARD

/*
    end of file generated by the tool \c hpp_generator
*/
